/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2011-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content

import java.lang.Object

import org.objectweb.fractal.api.control.IllegalLifeCycleException
import org.objectweb.fractal.api.control.LifeCycleController
import org.objectweb.fractal.julia.control.lifecycle.UseLifeCycleControllerTrait
import org.objectweb.fractal.julia.BasicControllerTrait

/**
 * Provides lifecycle related checks to a {@link SCAContentController}. Check
 * that the component is stopped and then call the overriden method.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.4.5
 */
trait LifeCycleSCAContentTrait extends BasicControllerTrait
with UseLifeCycleControllerTrait with SCAExtendedContentControllerTrait {

    // -----------------------------------------------------------------------
    // SCAIntentController
    // -----------------------------------------------------------------------

    override def setFcContentClass( c: Class[_] ) = {
        checkFcStopped
	    super.setFcContentClass(c)
    }
    
    override def setFcContent( content: Object ) = {      	
        checkFcStopped
	    super.setFcContent(content)
    }
    
    // -----------------------------------------------------------------------
    // Implementation specific
    // -----------------------------------------------------------------------

    private def checkFcStopped = {
        val state = weaveableLC.getFcState
        if( ! LifeCycleController.STOPPED.equals(state) ) {
        	val msg = "Component should be stopped"
        	throw new IllegalLifeCycleException(msg)
        }
    }    
}
